Package: testLibDynam
Title: Test package for loading precompiled DLL/SO placed within inst/libs/ 
License: GPL-2
Description:
Author: Hadley <h.wickham@gmail.com>
Maintainer: Hadley <h.wickham@gmail.com>
Version: 0.1
Collate: a.r
