Package: testShim
Title: Tools to make developing R code easier
License: GPL-2
Description: This package is for testing the devtools shim system.
Author: Hadley <h.wickham@gmail.com>
Maintainer: Hadley <h.wickham@gmail.com>
Version: 0.1
Collate: a.r
