Package: testHelp
Title: Tools to make developing R code easier
License: GPL-2
Description: Test package for devtools help.
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@rstudio.com", role = c("aut", "cre")),
    person("RStudio", role = "cph")
  )
Version: 0.1
RoxygenNote: 6.0.1
