Package: testLoadHelpers
Title: Tools to make developing R code easier
License: GPL-2
Description:
Author: Hadley <h.wickham@gmail.com>
Maintainer: Hadley <h.wickham@gmail.com>
Version: 0.1
Collate: a.r
Suggests: 
    testthat
RoxygenNote: 6.0.1.9000
Encoding: UTF-8
LazyData: true
