# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

rcpp_hello_world <- function() {
    .Call(`_testDllRcpp_rcpp_hello_world`)
}

rcpp_test_attributes <- function() {
    .Call(`_testDllRcpp_rcpp_test_attributes`)
}

