stopifnot(length(.dynLibs()) > 0)
pkgload::load_all("empty")
stopifnot(length(.dynLibs()) > 0)
gc()
stopifnot(length(.dynLibs()) > 0)
pkgload::load_all("empty")
stopifnot(length(.dynLibs()) > 0)
gc()
stopifnot(length(.dynLibs()) > 0)
