% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/testUnknownMacro.R
\name{testUnknownMacro}
\alias{testUnknownMacro}
\title{Topic}
\usage{
testUnknownMacro()
}
\description{
This macro is \unknown{}. This one is \known{}.
}
