% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/foofoo.r
\name{testCustomMacro}
\alias{testCustomMacro}
\title{Function that has a system and a custom macro}
\usage{
testCustomMacro()
}
\description{
This is \CRANpkg{pkgload}. \foobar{Yeah!}
}
