% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/foofoo.r
\name{testSysMacro}
\alias{testSysMacro}
\title{Function that has a system macro}
\usage{
testSysMacro()
}
\description{
This is \CRANpkg{pkgload}.
}
