% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/macro.R
\name{macro_downstream}
\alias{macro_downstream}
\title{Topic}
\usage{
macro_downstream()
}
\description{
This is a foreign macro \mjeqn{success}{success}.
}
